@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008, 2013, 2014 Apple Inc. All rights reserved.
+ * Copyright (C) 2008, 2013, 2014, 2015 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -33,17 +33,14 @@
 #include "Interpreter.h"
 #include "JSCInlines.h"
 #include "Options.h"
+#include <wtf/Lock.h>
 
 namespace JSC {
 
 #if !ENABLE(JIT)
 static size_t committedBytesCount = 0;
 
-static Mutex& stackStatisticsMutex()
-{
-    DEPRECATED_DEFINE_STATIC_LOCAL(Mutex, staticMutex, ());
-    return staticMutex;
-}    
+static StaticLock stackStatisticsMutex;
 #endif // !ENABLE(JIT)
 
 JSStack::JSStack(VM& vm)
@@ -97,8 +94,7 @@ bool JSStack::growSlowCase(Register* newTopOfStack)
     if (newCommitTop < reservationTop())
         return false;
 
-    // Otherwise, the growth is still within our budget. Go ahead and commit
-    // it and return true.
+    // Otherwise, the growth is still within our budget. Commit it and return true.
     m_reservation.commit(newCommitTop, delta);
     addToCommittedByteCount(delta);
     m_commitTop = newCommitTop;
@@ -140,14 +136,9 @@ void JSStack::releaseExcessCapacity()
     m_commitTop = highAddressWithReservedZone;
 }
 
-void JSStack::initializeThreading()
-{
-    stackStatisticsMutex();
-}
-
 void JSStack::addToCommittedByteCount(long byteCount)
 {
-    MutexLocker locker(stackStatisticsMutex());
+    LockHolder locker(stackStatisticsMutex);
     ASSERT(static_cast<long>(committedBytesCount) + byteCount > -1);
     committedBytesCount += byteCount;
 }
@@ -177,7 +168,7 @@ Register* JSStack::highAddress() const
 size_t JSStack::committedByteCount()
 {
 #if !ENABLE(JIT)
-    MutexLocker locker(stackStatisticsMutex());
+    LockHolder locker(stackStatisticsMutex);
     return committedBytesCount;
 #else
     // When using the C stack, we don't know how many stack pages are actually
