@@ -60,7 +60,9 @@ CSSSelector::CSSSelector(const QualifiedName& tagQName, bool tagIsForNamespaceRu
     , m_hasNameWithCase(false)
     , m_isForPage(false)
     , m_tagIsForNamespaceRule(tagIsForNamespaceRule)
+#if ENABLE(CSS_SELECTORS_LEVEL4)
     , m_descendantDoubleChildSyntax(false)
+#endif
     , m_caseInsensitiveAttributeValueMatching(false)
 {
     const AtomicString& tagLocalName = tagQName.localName();
@@ -82,7 +84,7 @@ void CSSSelector::createRareData()
     if (m_hasRareData)
         return;
     // Move the value to the rare data stucture.
-    m_data.m_rareData = RareData::create(adoptRef(m_data.m_value)).leakRef();
+    m_data.m_rareData = &RareData::create(adoptRef(m_data.m_value)).leakRef();
     m_hasRareData = true;
 }
 
@@ -372,7 +374,6 @@ static void appendPseudoClassFunctionTail(StringBuilder& str, const CSSSelector*
 
 }
 
-#if ENABLE(CSS_SELECTORS_LEVEL4)
 static void appendLangArgumentList(StringBuilder& str, const Vector<AtomicString>& argumentList)
 {
     unsigned argumentListSize = argumentList.size();
@@ -384,7 +385,6 @@ static void appendLangArgumentList(StringBuilder& str, const Vector<AtomicString
             str.appendLiteral(", ");
     }
 }
-#endif
 
 String CSSSelector::selectorText(const String& rightSide) const
 {
@@ -421,11 +421,9 @@ String CSSSelector::selectorText(const String& rightSide) const
                 str.append(')');
                 break;
             }
-#if ENABLE(CSS_SELECTORS_LEVEL4)
             case CSSSelector::PseudoClassAnyLink:
                 str.appendLiteral(":any-link");
                 break;
-#endif
             case CSSSelector::PseudoClassAnyLinkDeprecated:
                 str.appendLiteral(":-webkit-any-link");
                 break;
@@ -519,13 +517,9 @@ String CSSSelector::selectorText(const String& rightSide) const
                 break;
             case CSSSelector::PseudoClassLang:
                 str.appendLiteral(":lang(");
-#if ENABLE(CSS_SELECTORS_LEVEL4)
                 ASSERT_WITH_MESSAGE(cs->langArgumentList() && !cs->langArgumentList()->isEmpty(), "An empty :lang() is invalid and should never be generated by the parser.");
                 appendLangArgumentList(str, *cs->langArgumentList());
                 str.append(')');
-#else
-                appendPseudoClassFunctionTail(str, cs);
-#endif
                 break;
             case CSSSelector::PseudoClassLastChild:
                 str.appendLiteral(":last-child");
@@ -707,8 +701,10 @@ String CSSSelector::selectorText(const String& rightSide) const
     if (const CSSSelector* tagHistory = cs->tagHistory()) {
         switch (cs->relation()) {
         case CSSSelector::Descendant:
+#if ENABLE(CSS_SELECTORS_LEVEL4)
             if (cs->m_descendantDoubleChildSyntax)
                 return tagHistory->selectorText(" >> " + str.toString() + rightSide);
+#endif
             return tagHistory->selectorText(" " + str.toString() + rightSide);
         case CSSSelector::Child:
             return tagHistory->selectorText(" > " + str.toString() + rightSide);
@@ -741,13 +737,11 @@ void CSSSelector::setArgument(const AtomicString& value)
     m_data.m_rareData->m_argument = value;
 }
 
-#if ENABLE(CSS_SELECTORS_LEVEL4)
 void CSSSelector::setLangArgumentList(std::unique_ptr<Vector<AtomicString>> argumentList)
 {
     createRareData();
     m_data.m_rareData->m_langArgumentList = WTF::move(argumentList);
 }
-#endif
 
 void CSSSelector::setSelectorList(std::unique_ptr<CSSSelectorList> selectorList)
 {
